## $Id: customer.py 12400 2015-01-05 07:22:08Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Container for the various objects owned by customers.
"""
import grok
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from waeup.ikoba.utils.helpers import attrs_to_fields
from waeup.ikoba.customers.customer import Customer
from waeup.ikoba.customers.interfaces import ICustomerNavigation
from ikobacustom.pcn.customers.interfaces import IPCNCustomer
from ikobacustom.pcn.interfaces import MessageFactory as _


class PCNCustomer(Customer):
    """This is a customer container for the various objects
    owned by customers.
    """
    grok.implements(IPCNCustomer, ICustomerNavigation)
    grok.provides(IPCNCustomer)

    form_fields_interface = IPCNCustomer

PCNCustomer = attrs_to_fields(PCNCustomer)


class PCNCustomerFactory(grok.GlobalUtility):
    """A factory for customers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Customer')
    title = u"Create a new pcn customer.",
    description = u"This factory instantiates new pcn customer instances."

    def __call__(self, *args, **kw):
        return PCNCustomer()

    def getInterfaces(self):
        return implementedBy(PCNCustomer)
